var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import Debug from "debug";
import ImmutableTypes from "react-immutable-proptypes";
import React from "react";
import SlateTypes from "@zykj/slate-prop-types";
import logger from "slate-dev-logger";
import Types from "prop-types";
import Void from "./void";
import Text from "./text";
import getChildrenDecorations from "../utils/get-children-decorations";
/**
 * Debug.
 *
 * @type {Function}
 */
var debug = Debug("slate:node");
/**
 * Node.
 *
 * @type {Component}
 */
var Node = /** @class */ (function (_super) {
    __extends(Node, _super);
    function Node() {
        /**
         * Property types.
         *
         * @type {Object}
         */
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * Debug.
         *
         * @param {String} message
         * @param {Mixed} ...args
         */
        _this.debug = function (message) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var node = _this.props.node;
            var key = node.key, type = node.type;
            debug.apply(void 0, __spreadArrays([message, key + " (" + type + ")"], args));
        };
        /**
         * Render a `child` node.
         *
         * @param {Node} child
         * @param {Boolean} isSelected
         * @param {Array<Decoration>} decorations
         * @return {Element}
         */
        _this.renderNode = function (child, isSelected, decorations) {
            var _a = _this.props, block = _a.block, editor = _a.editor, node = _a.node, readOnly = _a.readOnly, isFocused = _a.isFocused;
            var Component = child.object == "text" ? Text : Node;
            return (React.createElement(Component, { block: node.object == "block" ? node : block, decorations: decorations, editor: editor, isSelected: isSelected, isFocused: isFocused && isSelected, key: child.key, node: child, parent: node, readOnly: readOnly }));
        };
        return _this;
    }
    /**
     * Should the node update?
     *
     * @param {Object} nextProps
     * @param {Object} value
     * @return {Boolean}
     */
    Node.prototype.shouldComponentUpdate = function (nextProps) {
        var props = this.props;
        var stack = props.editor.stack;
        var shouldUpdate = stack.$$find("shouldNodeComponentUpdate", props, nextProps);
        var n = nextProps;
        var p = props;
        // If the `Component` has a custom logic to determine whether the component
        // needs to be updated or not, return true if it returns true. If it returns
        // false, we need to ignore it, because it shouldn't be allowed it.
        if (shouldUpdate != null) {
            if (shouldUpdate) {
                return true;
            }
            if (shouldUpdate === false) {
                logger.warn("Returning false in `shouldNodeComponentUpdate` does not disable Slate's internal `shouldComponentUpdate` logic. If you want to prevent updates, use React's `shouldComponentUpdate` instead.");
            }
        }
        // If the `readOnly` status has changed, re-render in case there is any
        // user-land logic that depends on it, like nested editable contents.
        if (n.readOnly != p.readOnly)
            return true;
        // If the node has changed, update. PERF: There are cases where it will have
        // changed, but it's properties will be exactly the same (eg. copy-paste)
        // which this won't catch. But that's rare and not a drag on performance, so
        // for simplicity we just let them through.
        if (n.node != p.node)
            return true;
        // If the selection value of the node or of some of its children has changed,
        // re-render in case there is any user-land logic depends on it to render.
        // if the node is selected update it, even if it was already selected: the
        // selection value of some of its children could have been changed and they
        // need to be rendered again.
        if (n.isSelected || p.isSelected)
            return true;
        if (n.isFocused || p.isFocused)
            return true;
        // If the decorations have changed, update.
        if (!n.decorations.equals(p.decorations))
            return true;
        // Otherwise, don't update.
        return false;
    };
    /**
     * Render.
     *
     * @return {Element}
     */
    Node.prototype.render = function () {
        var _this = this;
        this.debug("render", this);
        var _a = this.props, editor = _a.editor, isSelected = _a.isSelected, isFocused = _a.isFocused, node = _a.node, decorations = _a.decorations, parent = _a.parent, readOnly = _a.readOnly;
        var value = editor.value;
        var selection = value.selection, schema = value.schema;
        var stack = editor.stack;
        var indexes = node.getSelectionIndexes(selection, isSelected);
        var decs = decorations.concat(node.getDecorations(stack));
        var childrenDecorations = getChildrenDecorations(node, decs);
        var children = [];
        node.nodes.forEach(function (child, i) {
            var isChildSelected = !!indexes && indexes.start <= i && i < indexes.end;
            children.push(_this.renderNode(child, isChildSelected, childrenDecorations[i]));
        });
        // Attributes that the developer must mix into the element in their
        // custom node renderer component.
        var attributes = { "data-key": node.key };
        // If it's a block node with inline children, add the proper `dir` attribute
        // for text direction.
        if (node.object == "block" && node.nodes.first().object != "block") {
            var direction = node.getTextDirection();
            if (direction == "rtl")
                attributes.dir = "rtl";
        }
        var props = {
            key: node.key,
            editor: editor,
            isFocused: isFocused,
            isSelected: isSelected,
            node: node,
            parent: parent,
            readOnly: readOnly
        };
        var placeholder = stack.$$find("renderPlaceholder", props);
        if (placeholder) {
            placeholder = React.cloneElement(placeholder, {
                key: node.key + "-placeholder"
            });
            children = __spreadArrays([placeholder], children);
        }
        var element = stack.$$find("renderNode", __assign(__assign({}, props), { attributes: attributes,
            children: children }));
        return schema.isVoid(node) ? (React.createElement(Void, __assign({}, this.props), element)) : (element);
    };
    Node.propTypes = {
        block: SlateTypes.block,
        decorations: ImmutableTypes.list.isRequired,
        editor: Types.object.isRequired,
        isFocused: Types.bool.isRequired,
        isSelected: Types.bool.isRequired,
        node: SlateTypes.node.isRequired,
        parent: SlateTypes.node.isRequired,
        readOnly: Types.bool.isRequired
    };
    return Node;
}(React.Component));
/**
 * Export.
 *
 * @type {Component}
 */
export default Node;
