import findDOMPoint from './find-dom-point';
/**
 * Find a native DOM range Slate `range`.
 *
 * @param {Range} range
 * @param {Window} win (optional)
 * @return {Object|Null}
 */
function findDOMRange(range, win) {
    if (win === void 0) { win = window; }
    var anchor = range.anchor, focus = range.focus, isBackward = range.isBackward, isCollapsed = range.isCollapsed;
    var domAnchor = findDOMPoint(anchor, win);
    var domFocus = isCollapsed ? domAnchor : findDOMPoint(focus, win);
    if (!domAnchor || !domFocus)
        return null;
    var r = win.document.createRange();
    var start = isBackward ? domFocus : domAnchor;
    var end = isBackward ? domAnchor : domFocus;
    r.setStart(start.node, start.offset);
    r.setEnd(end.node, end.offset);
    return r;
}
/**
 * Export.
 *
 * @type {Function}
 */
export default findDOMRange;
