import getWindow from "get-window";
import { IS_IE, IS_EDGE } from "@zykj/slate-dev-environment";
import findPoint from "./find-point";
import findDOMPoint from "./find-dom-point";
/**
 * Find a Slate range from a DOM `native` selection.
 *
 * @param {Selection} native
 * @param {Value} value
 * @return {Range}
 */
function findRange(native, value) {
    var el = native.anchorNode || native.startContainer;
    if (!el)
        return null;
    var window = getWindow(el);
    // If the `native` object is a DOM `Range` or `StaticRange` object, change it
    // into something that looks like a DOM `Selection` instead.
    if (native instanceof window.Range ||
        (window.StaticRange && native instanceof window.StaticRange)) {
        native = {
            anchorNode: native.startContainer,
            anchorOffset: native.startOffset,
            focusNode: native.endContainer,
            focusOffset: native.endOffset
        };
    }
    var anchorNode = native.anchorNode, anchorOffset = native.anchorOffset, focusNode = native.focusNode, focusOffset = native.focusOffset, isCollapsed = native.isCollapsed;
    var anchor = findPoint(anchorNode, anchorOffset, value);
    var focus = isCollapsed ? anchor : findPoint(focusNode, focusOffset, value);
    if (!anchor || !focus)
        return null;
    // COMPAT: ??? The Edge browser seems to have a case where if you select the
    // last word of a span, it sets the endContainer to the containing span.
    // `selection-is-backward` doesn't handle this case.
    if (IS_IE || IS_EDGE) {
        var domAnchor = findDOMPoint(anchor);
        var domFocus = findDOMPoint(focus);
        native = {
            anchorNode: domAnchor.node,
            anchorOffset: domAnchor.offset,
            focusNode: domFocus.node,
            focusOffset: domFocus.offset
        };
    }
    var document = value.document;
    var range = document.createRange({
        anchor: anchor,
        focus: focus
    });
    return range;
}
/**
 * Export.
 *
 * @type {Function}
 */
export default findRange;
