import React from "react";
import Types from "prop-types";
/**
 * Content.
 *
 * @type {Component}
 */
declare class Content extends React.Component<any, any> {
    element: any;
    /**
     * Property types.
     *
     * @type {Object}
     */
    static propTypes: {
        autoCorrect: Types.Validator<boolean>;
        children: Types.Validator<any>;
        className: Types.Requireable<string>;
        editor: Types.Validator<object>;
        readOnly: Types.Validator<boolean>;
        role: Types.Requireable<string>;
        spellCheck: Types.Validator<boolean>;
        style: Types.Requireable<object>;
        tabIndex: Types.Requireable<number>;
        tagName: Types.Requireable<string>;
    };
    /**
     * Default properties.
     *
     * @type {Object}
     */
    static defaultProps: {
        style: {};
        tagName: string;
    };
    /**
     * Temporary values.
     *
     * @type {Object}
     */
    tmp: {
        isUpdatingSelection: boolean;
    };
    /**
     * Create a set of bound event handlers.
     *
     * @type {Object}
     */
    handlers: any;
    onBlur: (event: any) => any;
    onFocus: (event: any) => any;
    onCompositionEnd: (event: any) => any;
    onCompositionStart: (event: any) => any;
    onCopy: (event: any) => any;
    onCut: (event: any) => any;
    onDragEnd: (event: any) => any;
    onDragOver: (event: any) => any;
    onDragStart: (event: any) => any;
    onDrop: (event: any) => any;
    onInput: (event: any) => any;
    onKeyDown: (event: any) => any;
    onKeyUp: (event: any) => any;
    onPaste: (event: any) => any;
    onSelect: (event: any) => any;
    /**
     * When the editor first mounts in the DOM we need to:
     *
     *   - Add native DOM event listeners.
     *   - Update the selection, in case it starts focused.
     */
    componentDidMount(): void;
    /**
     * When unmounting, remove DOM event listeners.
     */
    componentWillUnmount(): void;
    /**
     * On update, update the selection.
     */
    componentDidUpdate(): void;
    /**
     * Update the native DOM selection to reflect the internal model.
     */
    updateSelection: () => void;
    /**
     * The React ref method to set the root content element locally.
     *
     * @param {Element} element
     */
    ref: (element: any) => void;
    /**
     * Check if an event `target` is fired from within the contenteditable
     * element. This should be false for edits happening in non-contenteditable
     * children, such as void nodes and other nested Slate editors.
     *
     * @param {Element} target
     */
    isInEditor: (target: any) => boolean;
    /**
     * On `event` with `handler`.
     *
     * @param {String} handler
     * @param {Event} event
     */
    onEvent(handler: any, event: any): void;
    /**
     * On native `selectionchange` event, trigger the `onSelect` handler. This is
     * needed to account for React's `onSelect` being non-standard and not firing
     * until after a selection has been released. This causes issues in situations
     * where another change happens while a selection is being made.
     *
     * @param {Event} event
     */
    onNativeSelectionChange: any;
    /**
     * Render the editor content.
     *
     */
    render(): JSX.Element;
    /**
     * Render a `child` node of the document.
     *
     * @param {Node} child
     * @param {Boolean} isSelected
     */
    renderNode: (child: any, isSelected: any, decorations: any) => JSX.Element;
}
/**
 * Export.
 *
 * @type {Component}
 */
export default Content;
