import React from "react";
import Types from "prop-types";
import noop from "../utils/noop";
/**
 * Editor.
 *
 * @type {Component}
 */
declare class Editor extends React.Component<any, any> {
    /**
     * Property types.
     *
     * @type {Object}
     */
    static propTypes: {
        autoCorrect: Types.Requireable<boolean>;
        autoFocus: Types.Requireable<boolean>;
        className: Types.Requireable<string>;
        onChange: Types.Requireable<(...args: any[]) => any>;
        placeholder: Types.Requireable<any>;
        plugins: Types.Requireable<any[]>;
        readOnly: Types.Requireable<boolean>;
        role: Types.Requireable<string>;
        schema: Types.Requireable<object>;
        spellCheck: Types.Requireable<boolean>;
        style: Types.Requireable<object>;
        tabIndex: Types.Requireable<number>;
        value: (...args: any[]) => Error | null;
    };
    /**
     * Default properties.
     *
     * @type {Object}
     */
    static defaultProps: {
        autoFocus: boolean;
        autoCorrect: boolean;
        onChange: typeof noop;
        plugins: never[];
        readOnly: boolean;
        schema: {};
        spellCheck: boolean;
    };
    /**
     * Initial state.
     *
     * @type {Object}
     */
    state: {};
    /**
     * Temporary values.
     *
     * @type {Object}
     */
    tmp: {
        change: any;
        isChanging: boolean;
        operationsSize: null;
        plugins: null;
        resolves: number;
        updates: number;
        value: null;
    };
    /**
     * Create a set of bound event handlers.
     *
     * @param {Object}
     */
    handlers: {};
    /**
     * When the component first mounts, flush any temporary changes, and then,
     * focus the editor if `autoFocus` is set.
     */
    componentDidMount(): void;
    /**
     * When the component updates, flush any temporary change.
     */
    componentDidUpdate(prevProps: any): void;
    /**
     * Render the editor.
     *
     * @return {Element}
     */
    render(): any;
    /**
     * Get the editor's current plugins.
     *
     * @return {Array}
     */
    get plugins(): any;
    /**
     * Get the editor's current schema.
     *
     * @return {Schema}
     */
    get schema(): any;
    /**
     * Get the editor's current stack.
     *
     * @return {Stack}
     */
    get stack(): any;
    /**
     * Get the editor's current value.
     *
     * @return {Value}
     */
    get value(): any;
    /**
     * Perform a change on the editor, passing `...args` to `change.call`.
     *
     * @param {Mixed} ...args
     */
    change: (...args: any[]) => void;
    /**
     * Programmatically blur the editor.
     */
    blur: () => void;
    /**
     * Programmatically focus the editor.
     */
    focus: () => void;
    /**
     * On change.
     *
     * @param {Change} change
     */
    onChange: (change: any) => void;
    /**
     * On event.
     *
     * @param {String} handler
     * @param {Event} event
     */
    onEvent: (handler: any, event?: any) => void;
    /**
     * Resolve a change from the current `plugins`, a potential `change` and its
     * current operations `size`.
     *
     * @param {Array} plugins
     * @param {Change} change
     * @param {Number} size
     *
     */
    resolveChange: any;
    /**
     * Resolve a set of plugins from potential `plugins` and a `schema`.
     *
     * In addition to the plugins provided in props, this will initialize three
     * other plugins:
     *
     * - The top-level editor plugin, which allows for top-level handlers, etc.
     * - The two "core" plugins, one before all the other and one after.
     *
     * @param {Array|Void} plugins
     * @param {Schema|Object|Void} schema
     * @return {Array}
     */
    resolvePlugins: any;
    /**
     * Resolve a schema from the current `plugins`.
     *
     * @param {Array} plugins
     * @return {Schema}
     */
    resolveSchema: any;
    /**
     * Resolve a stack from the current `plugins`.
     *
     * @param {Array} plugins
     * @return {Stack}
     */
    resolveStack: any;
    /**
     * Resolve a value from the current `plugins` and a potential `value`.
     *
     * @param {Array} plugins
     * @param {Value} value
     * @return {Change}
     */
    resolveValue: any;
}
/**
 * Export.
 *
 * @type {Component}
 */
export default Editor;
