import _Object$defineProperty from "@babel/runtime-corejs3/core-js-stable/object/define-property";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _reduceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/reduce";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _Object$create from "@babel/runtime-corejs3/core-js-stable/object/create";
import _Object$setPrototypeOf from "@babel/runtime-corejs3/core-js-stable/object/set-prototype-of";

var __extends = this && this.__extends || function () {
  var extendStatics = function (d, b) {
    extendStatics = _Object$setPrototypeOf || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    };

    return extendStatics(d, b);
  };

  return function (d, b) {
    extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? _Object$create(b) : (__.prototype = b.prototype, new __());
  };
}();

var __assign = this && this.__assign || function () {
  __assign = _Object$assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var __spreadArrays = this && this.__spreadArrays || function () {
  for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;

  for (var r = Array(s), k = 0, i = 0; i < il; i++) for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++) r[k] = a[j];

  return r;
};

import Debug from "debug";
import Portal from "react-portal";
import React from "react";
import SlateTypes from "@zykj/slate-prop-types";
import Types from "prop-types";
import logger from "slate-dev-logger";
import { Schema, Stack } from "@zykj/slate";
import memoizeOne from "memoize-one";
import EVENT_HANDLERS from "../constants/event-handlers";
import PLUGINS_PROPS from "../constants/plugin-props";
import AfterPlugin from "../plugins/after";
import BeforePlugin from "../plugins/before";
import noop from "../utils/noop";
/**
 * Debug.
 *
 * @type {Function}
 */

var debug = Debug("slate:editor");
/**
 * Editor.
 *
 * @type {Component}
 */

var Editor =
/** @class */
function (_super) {
  __extends(Editor, _super);

  function Editor() {
    /**
     * Property types.
     *
     * @type {Object}
     */
    var _this = _super !== null && _super.apply(this, arguments) || this;
    /**
     * Initial state.
     *
     * @type {Object}
     */


    _this.state = {};
    /**
     * Temporary values.
     *
     * @type {Object}
     */

    _this.tmp = {
      change: null,
      isChanging: false,
      operationsSize: null,
      plugins: null,
      resolves: 0,
      updates: 0,
      value: null
    };
    /**
     * Create a set of bound event handlers.
     *
     * @param {Object}
     */

    _this.handlers = _reduceInstanceProperty(EVENT_HANDLERS).call(EVENT_HANDLERS, function (obj, handler) {
      obj[handler] = function (event) {
        return _this.onEvent(handler, event);
      };

      return obj;
    }, {});
    /**
     * Perform a change on the editor, passing `...args` to `change.call`.
     *
     * @param {Mixed} ...args
     */

    _this.change = function () {
      var args = [];

      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }

      if (_this.tmp.isChanging) {
        logger.warn("The `editor.change` method was called from within an existing `editor.change` callback. This is not allowed, and often due to calling `editor.change` directly from a plugin's event handler which is unnecessary.");
        return;
      }

      var change = _this.value.change();

      try {
        _this.tmp.isChanging = true;
        change.call.apply(change, args);
      } catch (error) {
        throw error;
      } finally {
        _this.tmp.isChanging = false;
      }

      _this.onChange(change);
    };
    /**
     * Programmatically blur the editor.
     */


    _this.blur = function () {
      _this.change(function (c) {
        return c.blur();
      });
    };
    /**
     * Programmatically focus the editor.
     */


    _this.focus = function () {
      _this.change(function (c) {
        return c.focus();
      });
    };
    /**
     * On change.
     *
     * @param {Change} change
     */


    _this.onChange = function (change) {
      // If the change doesn't define any operations to apply, abort.
      if (change.operations.size === 0) {
        return;
      }

      debug("onChange", {
        change: change
      });
      change = _this.resolveChange(_this.plugins, change, change.operations.size); // Store a reference to the last `value` and `plugins` that were seen by the
      // editor, so we can know whether to normalize a new unknown value if one
      // is passed in via `this.props`.

      _this.tmp.value = change.value;
      _this.tmp.plugins = _this.plugins; // Remove the temporary `change`, since it's being flushed.

      delete _this.tmp.change;
      delete _this.tmp.operationsSize;

      _this.props.onChange(change);
    };
    /**
     * On event.
     *
     * @param {String} handler
     * @param {Event} event
     */


    _this.onEvent = function (handler, event) {
      _this.change(function (change) {
        _this.stack.run(handler, event, change, _this);
      });
    };
    /**
     * Resolve a change from the current `plugins`, a potential `change` and its
     * current operations `size`.
     *
     * @param {Array} plugins
     * @param {Change} change
     * @param {Number} size
     *
     */


    _this.resolveChange = memoizeOne(function (plugins, change, size) {
      var stack = _this.resolveStack(plugins);

      stack.run("onChange", change, _this);
      return change;
    });
    /**
     * Resolve a set of plugins from potential `plugins` and a `schema`.
     *
     * In addition to the plugins provided in props, this will initialize three
     * other plugins:
     *
     * - The top-level editor plugin, which allows for top-level handlers, etc.
     * - The two "core" plugins, one before all the other and one after.
     *
     * @param {Array|Void} plugins
     * @param {Schema|Object|Void} schema
     * @return {Array}
     */

    _this.resolvePlugins = memoizeOne(function (plugins, schema) {
      if (plugins === void 0) {
        plugins = [];
      }

      if (schema === void 0) {
        schema = {};
      }

      debug("resolvePlugins", {
        plugins: plugins,
        schema: schema
      });
      _this.tmp.resolves++;
      var beforePlugin = BeforePlugin();
      var afterPlugin = AfterPlugin();
      var editorPlugin = {
        schema: schema
      };

      var _loop_1 = function (prop) {
        // Skip `onChange` because the editor's `onChange` is special.
        if (prop == "onChange") return "continue"; // Skip `schema` because it can't be proxied easily, so it must be passed
        // in as an argument to this function instead.

        if (prop == "schema") return "continue"; // Define a function that will just proxies into `props`.

        editorPlugin[prop] = function () {
          var _a;

          var args = [];

          for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
          }

          return _this.props[prop] && (_a = _this.props)[prop].apply(_a, args);
        };
      };

      for (var _i = 0, PLUGINS_PROPS_1 = PLUGINS_PROPS; _i < PLUGINS_PROPS_1.length; _i++) {
        var prop = PLUGINS_PROPS_1[_i];

        _loop_1(prop);
      }

      return __spreadArrays([beforePlugin, editorPlugin], plugins, [afterPlugin]);
    });
    /**
     * Resolve a schema from the current `plugins`.
     *
     * @param {Array} plugins
     * @return {Schema}
     */

    _this.resolveSchema = memoizeOne(function (plugins) {
      debug("resolveSchema", {
        plugins: plugins
      });
      var schema = Schema.create({
        plugins: plugins
      });
      return schema;
    });
    /**
     * Resolve a stack from the current `plugins`.
     *
     * @param {Array} plugins
     * @return {Stack}
     */

    _this.resolveStack = memoizeOne(function (plugins) {
      debug("resolveStack", {
        plugins: plugins
      });
      var stack = Stack.create({
        plugins: plugins
      });
      return stack;
    });
    /**
     * Resolve a value from the current `plugins` and a potential `value`.
     *
     * @param {Array} plugins
     * @param {Value} value
     * @return {Change}
     */

    _this.resolveValue = memoizeOne(function (plugins, value) {
      debug("resolveValue", {
        plugins: plugins,
        value: value
      });
      var change = value.change();
      change = _this.resolveChange(plugins, change, change.operations.size); // Store the change and it's operations count so that it can be flushed once
      // the component next updates.

      _this.tmp.change = change;
      _this.tmp.operationsSize = change.operations.size;
      return change.value;
    });
    return _this;
  }
  /**
   * When the component first mounts, flush any temporary changes, and then,
   * focus the editor if `autoFocus` is set.
   */


  Editor.prototype.componentDidMount = function () {
    // Increment the updates counter as a baseline.
    this.tmp.updates++;
    var autoFocus = this.props.autoFocus;
    var change = this.tmp.change;

    if (autoFocus) {
      if (change) {
        change.focus();
      } else {
        this.focus();
      }
    }

    if (change) {
      this.onChange(change);
    }
  };
  /**
   * When the component updates, flush any temporary change.
   */


  Editor.prototype.componentDidUpdate = function (prevProps) {
    this.tmp.updates++;
    var _a = this.tmp,
        change = _a.change,
        resolves = _a.resolves,
        updates = _a.updates; // If we've resolved a few times already, and it's exactly in line with
    // the updates, then warn the user that they may be doing something wrong.

    if (resolves > 5 && resolves === updates) {
      logger.warn("A Slate <Editor> component is re-resolving `props.plugins` or `props.schema` on each update, which leads to poor performance. This is often due to passing in a new `schema` or `plugins` prop with each render by declaring them inline in your render function. Do not do this!");
    }

    if (change) {
      this.onChange(change);
    }
  };
  /**
   * Render the editor.
   *
   * @return {Element}
   */


  Editor.prototype.render = function () {
    var _context;

    debug("render", this);

    var children = _mapInstanceProperty(_context = this.stack.$$map("renderPortal", this.value, this)).call(_context, function (child, i) {
      return /*#__PURE__*/React.createElement(Portal, {
        key: i,
        isOpened: true
      }, child);
    });

    var props = __assign(__assign({}, this.props), {
      children: children
    });

    var tree = this.stack.render("renderEditor", props, this);
    return tree;
  };

  _Object$defineProperty(Editor.prototype, "plugins", {
    /**
     * Get the editor's current plugins.
     *
     * @return {Array}
     */
    get: function () {
      var plugins = this.resolvePlugins(this.props.plugins, this.props.schema);
      return plugins;
    },
    enumerable: false,
    configurable: true
  });

  _Object$defineProperty(Editor.prototype, "schema", {
    /**
     * Get the editor's current schema.
     *
     * @return {Schema}
     */
    get: function () {
      var schema = this.resolveSchema(this.plugins);
      return schema;
    },
    enumerable: false,
    configurable: true
  });

  _Object$defineProperty(Editor.prototype, "stack", {
    /**
     * Get the editor's current stack.
     *
     * @return {Stack}
     */
    get: function () {
      var stack = this.resolveStack(this.plugins);
      return stack;
    },
    enumerable: false,
    configurable: true
  });

  _Object$defineProperty(Editor.prototype, "value", {
    /**
     * Get the editor's current value.
     *
     * @return {Value}
     */
    get: function () {
      // If the current `plugins` and `value` are the same as the last seen ones
      // that were saved in `tmp`, don't re-resolve because that will trigger
      // extra `onChange` runs.
      if (this.plugins === this.tmp.plugins && this.props.value === this.tmp.value) {
        return this.tmp.value;
      }

      var value = this.resolveValue(this.plugins, this.props.value);
      return value;
    },
    enumerable: false,
    configurable: true
  });

  Editor.propTypes = {
    autoCorrect: Types.bool,
    autoFocus: Types.bool,
    className: Types.string,
    onChange: Types.func,
    placeholder: Types.any,
    plugins: Types.array,
    readOnly: Types.bool,
    role: Types.string,
    schema: Types.object,
    spellCheck: Types.bool,
    style: Types.object,
    tabIndex: Types.number,
    value: SlateTypes.value.isRequired
  };
  /**
   * Default properties.
   *
   * @type {Object}
   */

  Editor.defaultProps = {
    autoFocus: false,
    autoCorrect: true,
    onChange: noop,
    plugins: [],
    readOnly: false,
    schema: {},
    spellCheck: true
  };
  return Editor;
}(React.Component);
/**
 * Mix in the prop types for the event handlers.
 */


for (var _i = 0, EVENT_HANDLERS_1 = EVENT_HANDLERS; _i < EVENT_HANDLERS_1.length; _i++) {
  var prop = EVENT_HANDLERS_1[_i];
  Editor.propTypes[prop] = Types.func;
}
/**
 * Export.
 *
 * @type {Component}
 */


export default Editor;