import findDOMNode from "./find-dom-node";
/**
 * Find a native DOM selection point from a Slate `point`.
 *
 * @param {Point} point
 * @param {Window} win (optional)
 * @return {Object|Null}
 */

function findDOMPoint(point, win) {
  if (win === void 0) {
    win = window;
  }

  var el = findDOMNode(point.key, win);
  var start = 0;
  var n; // COMPAT: In IE, this method's arguments are not optional, so we have to
  // pass in all four even though the last two are defaults. (2017/10/25)

  var iterator = win.document.createNodeIterator(el, NodeFilter.SHOW_TEXT, function () {
    return NodeFilter.FILTER_ACCEPT;
  });

  while (n = iterator.nextNode()) {
    var length_1 = n.textContent.length;
    var end = start + length_1;

    if (point.offset <= end) {
      var o = point.offset - start;
      return {
        node: n,
        offset: o >= 0 ? o : 0
      };
    }

    start = end;
  }

  return null;
}
/**
 * Export.
 *
 * @type {Function}
 */


export default findDOMPoint;