"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _setTimeout2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set-timeout"));

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _create = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/create"));

var _setPrototypeOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/set-prototype-of"));

var _debug = _interopRequireDefault(require("debug"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _getWindow = _interopRequireDefault(require("get-window"));

var _slateDevEnvironment = require("@zykj/slate-dev-environment");

var _slateDevLogger = _interopRequireDefault(require("slate-dev-logger"));

var _throttle = _interopRequireDefault(require("lodash/throttle"));

var _eventHandlers = _interopRequireDefault(require("../constants/event-handlers"));

var _node = _interopRequireDefault(require("./node"));

var _findDomRange = _interopRequireDefault(require("../utils/find-dom-range"));

var _findRange = _interopRequireDefault(require("../utils/find-range"));

var _getChildrenDecorations = _interopRequireDefault(require("../utils/get-children-decorations"));

var _scrollToSelection = _interopRequireDefault(require("../utils/scroll-to-selection"));

var _removeAllRanges = _interopRequireDefault(require("../utils/remove-all-ranges"));

var __extends = void 0 && (void 0).__extends || function () {
  var extendStatics = function (d, b) {
    extendStatics = _setPrototypeOf.default || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    };

    return extendStatics(d, b);
  };

  return function (d, b) {
    extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? (0, _create.default)(b) : (__.prototype = b.prototype, new __());
  };
}();

var __assign = void 0 && (void 0).__assign || function () {
  __assign = _assign.default || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

/**
 * Debug.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:content");
/**
 * Content.
 *
 * @type {Component}
 */

var Content =
/** @class */
function (_super) {
  __extends(Content, _super);

  function Content() {
    var _this = _super !== null && _super.apply(this, arguments) || this;
    /**
     * Temporary values.
     *
     * @type {Object}
     */


    _this.tmp = {
      isUpdatingSelection: false
    };
    /**
     * Create a set of bound event handlers.
     *
     * @type {Object}
     */

    _this.handlers = (0, _reduce.default)(_eventHandlers.default).call(_eventHandlers.default, function (obj, handler) {
      obj[handler] = function (event) {
        return _this.onEvent(handler, event);
      };

      return obj;
    }, {});
    /**
     * Update the native DOM selection to reflect the internal model.
     */

    _this.updateSelection = function () {
      var editor = _this.props.editor;
      var value = editor.value;
      var selection = value.selection;
      var isBackward = selection.isBackward;
      var window = (0, _getWindow.default)(_this.element);
      var native = window.getSelection(); // .getSelection() can return null in some cases
      // https://bugzilla.mozilla.org/show_bug.cgi?id=827585

      if (!native) return;
      var rangeCount = native.rangeCount,
          anchorNode = native.anchorNode; // .getSelection() can return null in some cases
      // https://bugzilla.mozilla.org/show_bug.cgi?id=827585

      if (!native) return;

      if (!native || editor.state.isComposing) {
        return;
      } // If both selections are blurred, do nothing.


      if (!rangeCount && selection.isBlurred) return; // If the selection has been blurred, but is still inside the editor in the
      // DOM, blur it manually.

      if (selection.isBlurred) {
        if (!_this.isInEditor(anchorNode)) return;
        (0, _removeAllRanges.default)(native);

        _this.element.blur();

        debug("updateSelection", {
          selection: selection,
          native: native
        });
        return;
      } // If the selection isn't set, do nothing.


      if (selection.isUnset) return; // Otherwise, figure out which DOM nodes should be selected...

      var current = !!rangeCount && native.getRangeAt(0);
      var range = (0, _findDomRange.default)(selection, window);

      if (!range) {
        _slateDevLogger.default.error("Unable to find a native DOM range from the current selection.", {
          selection: selection
        });

        return;
      }

      var startContainer = range.startContainer,
          startOffset = range.startOffset,
          endContainer = range.endContainer,
          endOffset = range.endOffset; // If the new range matches the current selection, there is nothing to fix.
      // COMPAT: The native `Range` object always has it's "start" first and "end"
      // last in the DOM. It has no concept of "backwards/forwards", so we have
      // to check both orientations here. (2017/10/31)

      if (current) {
        if (startContainer == current.startContainer && startOffset == current.startOffset && endContainer == current.endContainer && endOffset == current.endOffset || startContainer == current.endContainer && startOffset == current.endOffset && endContainer == current.startContainer && endOffset == current.startOffset) {
          return;
        }
      } // Otherwise, set the `isUpdatingSelection` flag and update the selection.


      _this.tmp.isUpdatingSelection = true;
      (0, _removeAllRanges.default)(native); // COMPAT: IE 11 does not support Selection.setBaseAndExtent

      if (native.setBaseAndExtent) {
        // COMPAT: Since the DOM range has no concept of backwards/forwards
        // we need to check and do the right thing here.
        if (isBackward) {
          native.setBaseAndExtent(range.endContainer, range.endOffset, range.startContainer, range.startOffset);
        } else {
          native.setBaseAndExtent(range.startContainer, range.startOffset, range.endContainer, range.endOffset);
        }
      } else {
        // COMPAT: IE 11 does not support Selection.extend, fallback to addRange
        native.addRange(range);
      } // Scroll to the selection, in case it's out of view.


      (0, _scrollToSelection.default)(native); // Then unset the `isUpdatingSelection` flag after a delay.

      (0, _setTimeout2.default)(function () {
        // COMPAT: In Firefox, it's not enough to create a range, you also need to
        // focus the contenteditable element too. (2016/11/16)
        if (_slateDevEnvironment.IS_FIREFOX && _this.element) _this.element.focus();
        _this.tmp.isUpdatingSelection = false;
      });
      debug("updateSelection", {
        selection: selection,
        native: native
      });
    };
    /**
     * The React ref method to set the root content element locally.
     *
     * @param {Element} element
     */


    _this.ref = function (element) {
      _this.element = element;
    };
    /**
     * Check if an event `target` is fired from within the contenteditable
     * element. This should be false for edits happening in non-contenteditable
     * children, such as void nodes and other nested Slate editors.
     *
     * @param {Element} target
     */


    _this.isInEditor = function (target) {
      var element = _this.element; // COMPAT: Text nodes don't have `isContentEditable` property. So, when
      // `target` is a text node use its parent node for check.

      var el = target.nodeType === 3 ? target.parentNode : target;
      return el.isContentEditable && (el === element || el.closest("[data-slate-editor]") === element);
    };
    /**
     * On native `selectionchange` event, trigger the `onSelect` handler. This is
     * needed to account for React's `onSelect` being non-standard and not firing
     * until after a selection has been released. This causes issues in situations
     * where another change happens while a selection is being made.
     *
     * @param {Event} event
     */


    _this.onNativeSelectionChange = (0, _throttle.default)(function (event) {
      if (_this.props.readOnly) return;
      var window = (0, _getWindow.default)(event.target);
      var activeElement = window.document.activeElement;
      if (activeElement !== _this.element) return;

      _this.props.onSelect(event);
    }, 100);
    /**
     * Render a `child` node of the document.
     *
     * @param {Node} child
     * @param {Boolean} isSelected
     */

    _this.renderNode = function (child, isSelected, decorations) {
      var _a = _this.props,
          editor = _a.editor,
          readOnly = _a.readOnly;
      var value = editor.value;
      var document = value.document,
          selection = value.selection;
      var isFocused = selection.isFocused;
      return /*#__PURE__*/_react.default.createElement(_node.default, {
        block: null,
        editor: editor,
        decorations: decorations,
        isSelected: isSelected,
        isFocused: isFocused && isSelected,
        key: child.key,
        node: child,
        parent: document,
        readOnly: readOnly
      });
    };

    return _this;
  }
  /**
   * When the editor first mounts in the DOM we need to:
   *
   *   - Add native DOM event listeners.
   *   - Update the selection, in case it starts focused.
   */


  Content.prototype.componentDidMount = function () {
    var window = (0, _getWindow.default)(this.element);
    window.document.addEventListener("selectionchange", this.onNativeSelectionChange); // COMPAT: Restrict scope of `beforeinput` to clients that support the
    // Input Events Level 2 spec, since they are preventable events.

    if (_slateDevEnvironment.HAS_INPUT_EVENTS_LEVEL_2) {
      // @ts-ignore
      this.element.addEventListener("beforeinput", this.handlers.onBeforeInput);
    }

    this.updateSelection();
  };
  /**
   * When unmounting, remove DOM event listeners.
   */


  Content.prototype.componentWillUnmount = function () {
    var window = (0, _getWindow.default)(this.element);

    if (window) {
      window.document.removeEventListener("selectionchange", this.onNativeSelectionChange);
    } // COMPAT: Restrict scope of `beforeinput` to mobile.


    if (_slateDevEnvironment.HAS_INPUT_EVENTS_LEVEL_2) {
      this.element.removeEventListener("beforeinput", this.handlers.onBeforeInput);
    }
  };
  /**
   * On update, update the selection.
   */


  Content.prototype.componentDidUpdate = function () {
    this.updateSelection();
  };
  /**
   * On `event` with `handler`.
   *
   * @param {String} handler
   * @param {Event} event
   */


  Content.prototype.onEvent = function (handler, event) {
    debug("onEvent", handler); // Ignore `onBlur`, `onFocus` and `onSelect` events generated
    // programmatically while updating selection.

    if (this.tmp.isUpdatingSelection && (handler == "onSelect" || handler == "onBlur" || handler == "onFocus")) {
      return;
    } // COMPAT: There are situations where a select event will fire with a new
    // native selection that resolves to the same internal position. In those
    // cases we don't need to trigger any changes, since our internal model is
    // already up to date, but we do want to update the native selection again
    // to make sure it is in sync. (2017/10/16)


    if (handler == "onSelect") {
      var editor = this.props.editor;
      var value = editor.value;
      var selection = value.selection;
      var window_1 = (0, _getWindow.default)(event.target);
      var native = window_1.getSelection();
      var range = (0, _findRange.default)(native, value);

      if (range && range.equals(selection.toRange())) {
        this.updateSelection();
        return;
      }
    } // Don't handle drag and drop events coming from embedded editors.


    if (handler == "onDragEnd" || handler == "onDragEnter" || handler == "onDragExit" || handler == "onDragLeave" || handler == "onDragOver" || handler == "onDragStart" || handler == "onDrop") {
      var target = event.target;
      var targetEditorNode = target.closest("[data-slate-editor]");
      if (targetEditorNode !== this.element) return;
    } // Some events require being in editable in the editor, so if the event
    // target isn't, ignore them.


    if (handler == "onBeforeInput" || handler == "onBlur" || handler == "onCompositionEnd" || handler == "onCompositionStart" || handler == "onCopy" || handler == "onCut" || handler == "onFocus" || handler == "onInput" || handler == "onKeyDown" || handler == "onKeyUp" || handler == "onPaste" || handler == "onSelect") {
      if (!this.isInEditor(event.target)) return;
    }

    this.props[handler](event);
  };
  /**
   * Render the editor content.
   *
   */


  Content.prototype.render = function () {
    var _context, _context2;

    var _this = this;

    var _a = this,
        props = _a.props,
        handlers = _a.handlers;

    var className = props.className,
        readOnly = props.readOnly,
        editor = props.editor,
        tabIndex = props.tabIndex,
        role = props.role,
        tagName = props.tagName,
        spellCheck = props.spellCheck;
    var value = editor.value,
        stack = editor.stack;
    var Container = tagName;
    var document = value.document,
        selection = value.selection,
        decorations = value.decorations;
    var indexes = document.getSelectionIndexes(selection);
    var decs = (0, _concat.default)(_context = document.getDecorations(stack)).call(_context, decorations);
    var childrenDecorations = (0, _getChildrenDecorations.default)(document, decs);
    var children = (0, _map.default)(_context2 = document.nodes.toArray()).call(_context2, function (child, i) {
      var isSelected = !!indexes && indexes.start <= i && i < indexes.end;
      return _this.renderNode(child, isSelected, childrenDecorations[i]);
    });

    var style = __assign(__assign({
      // Prevent the default outline styles.
      outline: "none",
      // Preserve adjacent whitespace and new lines.
      whiteSpace: "pre-wrap",
      // Allow words to break if they are too long.
      wordWrap: "break-word"
    }, readOnly ? {} : {
      WebkitUserModify: "read-write-plaintext-only"
    }), props.style);

    debug("render", {
      props: props
    });
    return /*#__PURE__*/_react.default.createElement(Container, __assign({}, handlers, {
      "data-slate-editor": true,
      ref: this.ref,
      "data-key": document.key,
      contentEditable: readOnly ? null : true,
      suppressContentEditableWarning: true,
      className: className,
      autoCorrect: props.autoCorrect ? "on" : "off",
      spellCheck: spellCheck,
      style: style,
      role: readOnly ? null : role || "textbox",
      tabIndex: tabIndex,
      "data-gramm": false
    }), children, this.props.children);
  };
  /**
   * Property types.
   *
   * @type {Object}
   */


  Content.propTypes = {
    autoCorrect: _propTypes.default.bool.isRequired,
    children: _propTypes.default.any.isRequired,
    className: _propTypes.default.string,
    editor: _propTypes.default.object.isRequired,
    readOnly: _propTypes.default.bool.isRequired,
    role: _propTypes.default.string,
    spellCheck: _propTypes.default.bool.isRequired,
    style: _propTypes.default.object,
    tabIndex: _propTypes.default.number,
    tagName: _propTypes.default.string
  };
  /**
   * Default properties.
   *
   * @type {Object}
   */

  Content.defaultProps = {
    style: {},
    tagName: "div"
  };
  return Content;
}(_react.default.Component);
/**
 * Mix in handler prop types.
 */


(0, _forEach.default)(_eventHandlers.default).call(_eventHandlers.default, function (handler) {
  Content.propTypes[handler] = _propTypes.default.func.isRequired;
});
/**
 * Export.
 *
 * @type {Component}
 */

var _default = Content;
exports.default = _default;