"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));

var _create = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/create"));

var _setPrototypeOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/set-prototype-of"));

var _debug = _interopRequireDefault(require("debug"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _slatePropTypes = _interopRequireDefault(require("@zykj/slate-prop-types"));

var _offsetKey = _interopRequireDefault(require("../utils/offset-key"));

var __extends = void 0 && (void 0).__extends || function () {
  var extendStatics = function (d, b) {
    extendStatics = _setPrototypeOf.default || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    };

    return extendStatics(d, b);
  };

  return function (d, b) {
    extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? (0, _create.default)(b) : (__.prototype = b.prototype, new __());
  };
}();

var __spreadArrays = void 0 && (void 0).__spreadArrays || function () {
  for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;

  for (var r = Array(s), k = 0, i = 0; i < il; i++) for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++) r[k] = a[j];

  return r;
};

/**
 * Debugger.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:leaves");
/**
 * Leaf.
 *
 * @type {Component}
 */

var Leaf =
/** @class */
function (_super) {
  __extends(Leaf, _super);

  function Leaf() {
    /**
     * Property types.
     *
     * @type {Object}
     */
    var _this = _super !== null && _super.apply(this, arguments) || this;
    /**
     * Debug.
     *
     * @param {String} message
     * @param {Mixed} ...args
     */


    _this.debug = function (message) {
      var args = [];

      for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
      }

      debug.apply(void 0, __spreadArrays([message, _this.props.node.key + "-" + _this.props.index], args));
    };

    return _this;
  }
  /**
   * Should component update?
   *
   * @param {Object} props
   * @return {Boolean}
   */


  Leaf.prototype.shouldComponentUpdate = function (props) {
    // If any of the regular properties have changed, re-render.
    if (props.index != this.props.index || props.marks != this.props.marks || props.text != this.props.text || props.parent != this.props.parent) {
      return true;
    } // Otherwise, don't update.


    return false;
  };
  /**
   * Render the leaf.
   *
   * @return {Element}
   */


  Leaf.prototype.render = function () {
    this.debug("render", this);
    var _a = this.props,
        node = _a.node,
        index = _a.index;

    var offsetKey = _offsetKey.default.stringify({
      key: node.key,
      index: index
    });

    return /*#__PURE__*/_react.default.createElement("span", {
      "data-offset-key": offsetKey
    }, this.renderMarks());
  };
  /**
   * Render all of the leaf's mark components.
   *
   * @return {Element}
   */


  Leaf.prototype.renderMarks = function () {
    var _a = this.props,
        marks = _a.marks,
        node = _a.node,
        offset = _a.offset,
        text = _a.text,
        editor = _a.editor;
    var stack = editor.stack;
    var leaf = this.renderText();
    var attributes = {
      "data-slate-leaf": true
    };
    return (0, _reduce.default)(marks).call(marks, function (children, mark) {
      var props = {
        editor: editor,
        mark: mark,
        marks: marks,
        node: node,
        offset: offset,
        text: text,
        children: children,
        attributes: attributes
      };
      var element = stack.$$find("renderMark", props);
      return element || children;
    }, leaf);
  };
  /**
   * Render the text content of the leaf, accounting for browsers.
   *
   * @return {Element}
   */


  Leaf.prototype.renderText = function () {
    var _a = this.props,
        block = _a.block,
        node = _a.node,
        editor = _a.editor,
        parent = _a.parent,
        text = _a.text,
        index = _a.index,
        leaves = _a.leaves;
    var value = editor.value;
    var schema = value.schema; // COMPAT: Render text inside void nodes with a zero-width space.
    // So the node can contain selection but the text is not visible.

    if (schema.isVoid(parent)) {
      return /*#__PURE__*/_react.default.createElement("span", {
        "data-slate-zero-width": "z"
      }, "\u200B");
    } // COMPAT: If this is the last text node in an empty block, render a zero-
    // width space that will convert into a line break when copying and pasting
    // to support expected plain text.


    if (text === "" && parent.object === "block" && parent.text === "" && parent.nodes.size === 1) {
      return /*#__PURE__*/_react.default.createElement("span", {
        "data-slate-zero-width": "n"
      }, "\u200B");
    } // COMPAT: If the text is empty, it's because it's on the edge of an inline
    // void node, so we render a zero-width space so that the selection can be
    // inserted next to it still.


    if (text === "") {
      return /*#__PURE__*/_react.default.createElement("span", {
        "data-slate-zero-width": "z"
      }, "\u200B");
    } // COMPAT: Browsers will collapse trailing new lines at the end of blocks,
    // so we need to add an extra trailing new lines to prevent that.


    var lastText = block.getLastText();
    var lastChar = text.charAt(text.length - 1);
    var isLastText = node === lastText;
    var isLastLeaf = index === leaves.size - 1;
    if (isLastText && isLastLeaf && lastChar === "\n") return text + "\n"; // Otherwise, just return the text.

    return text;
  };

  Leaf.propTypes = {
    block: _slatePropTypes.default.block.isRequired,
    editor: _propTypes.default.object.isRequired,
    index: _propTypes.default.number.isRequired,
    leaves: _slatePropTypes.default.leaves.isRequired,
    marks: _slatePropTypes.default.marks.isRequired,
    node: _slatePropTypes.default.node.isRequired,
    offset: _propTypes.default.number.isRequired,
    parent: _slatePropTypes.default.node.isRequired,
    text: _propTypes.default.string.isRequired
  };
  return Leaf;
}(_react.default.Component);
/**
 * Export.
 *
 * @type {Component}
 */


var _default = Leaf;
exports.default = _default;