"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var _create = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/create"));

var _setPrototypeOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/set-prototype-of"));

var _debug = _interopRequireDefault(require("debug"));

var _reactImmutableProptypes = _interopRequireDefault(require("react-immutable-proptypes"));

var _react = _interopRequireDefault(require("react"));

var _slatePropTypes = _interopRequireDefault(require("@zykj/slate-prop-types"));

var _slateDevLogger = _interopRequireDefault(require("slate-dev-logger"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _void = _interopRequireDefault(require("./void"));

var _text = _interopRequireDefault(require("./text"));

var _getChildrenDecorations = _interopRequireDefault(require("../utils/get-children-decorations"));

var __extends = void 0 && (void 0).__extends || function () {
  var extendStatics = function (d, b) {
    extendStatics = _setPrototypeOf.default || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    };

    return extendStatics(d, b);
  };

  return function (d, b) {
    extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? (0, _create.default)(b) : (__.prototype = b.prototype, new __());
  };
}();

var __assign = void 0 && (void 0).__assign || function () {
  __assign = _assign.default || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var __spreadArrays = void 0 && (void 0).__spreadArrays || function () {
  for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;

  for (var r = Array(s), k = 0, i = 0; i < il; i++) for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++) r[k] = a[j];

  return r;
};

/**
 * Debug.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:node");
/**
 * Node.
 *
 * @type {Component}
 */

var Node =
/** @class */
function (_super) {
  __extends(Node, _super);

  function Node() {
    /**
     * Property types.
     *
     * @type {Object}
     */
    var _this = _super !== null && _super.apply(this, arguments) || this;
    /**
     * Debug.
     *
     * @param {String} message
     * @param {Mixed} ...args
     */


    _this.debug = function (message) {
      var args = [];

      for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
      }

      var node = _this.props.node;
      var key = node.key,
          type = node.type;
      debug.apply(void 0, __spreadArrays([message, key + " (" + type + ")"], args));
    };
    /**
     * Render a `child` node.
     *
     * @param {Node} child
     * @param {Boolean} isSelected
     * @param {Array<Decoration>} decorations
     * @return {Element}
     */


    _this.renderNode = function (child, isSelected, decorations) {
      var _a = _this.props,
          block = _a.block,
          editor = _a.editor,
          node = _a.node,
          readOnly = _a.readOnly,
          isFocused = _a.isFocused;
      var Component = child.object == "text" ? _text.default : Node;
      return /*#__PURE__*/_react.default.createElement(Component, {
        block: node.object == "block" ? node : block,
        decorations: decorations,
        editor: editor,
        isSelected: isSelected,
        isFocused: isFocused && isSelected,
        key: child.key,
        node: child,
        parent: node,
        readOnly: readOnly
      });
    };

    return _this;
  }
  /**
   * Should the node update?
   *
   * @param {Object} nextProps
   * @param {Object} value
   * @return {Boolean}
   */


  Node.prototype.shouldComponentUpdate = function (nextProps) {
    var props = this.props;
    var stack = props.editor.stack;
    var shouldUpdate = stack.$$find("shouldNodeComponentUpdate", props, nextProps);
    var n = nextProps;
    var p = props; // If the `Component` has a custom logic to determine whether the component
    // needs to be updated or not, return true if it returns true. If it returns
    // false, we need to ignore it, because it shouldn't be allowed it.

    if (shouldUpdate != null) {
      if (shouldUpdate) {
        return true;
      }

      if (shouldUpdate === false) {
        _slateDevLogger.default.warn("Returning false in `shouldNodeComponentUpdate` does not disable Slate's internal `shouldComponentUpdate` logic. If you want to prevent updates, use React's `shouldComponentUpdate` instead.");
      }
    } // If the `readOnly` status has changed, re-render in case there is any
    // user-land logic that depends on it, like nested editable contents.


    if (n.readOnly != p.readOnly) return true; // If the node has changed, update. PERF: There are cases where it will have
    // changed, but it's properties will be exactly the same (eg. copy-paste)
    // which this won't catch. But that's rare and not a drag on performance, so
    // for simplicity we just let them through.

    if (n.node != p.node) return true; // If the selection value of the node or of some of its children has changed,
    // re-render in case there is any user-land logic depends on it to render.
    // if the node is selected update it, even if it was already selected: the
    // selection value of some of its children could have been changed and they
    // need to be rendered again.

    if (n.isSelected || p.isSelected) return true;
    if (n.isFocused || p.isFocused) return true; // If the decorations have changed, update.

    if (!n.decorations.equals(p.decorations)) return true; // Otherwise, don't update.

    return false;
  };
  /**
   * Render.
   *
   * @return {Element}
   */


  Node.prototype.render = function () {
    var _context;

    var _this = this;

    this.debug("render", this);
    var _a = this.props,
        editor = _a.editor,
        isSelected = _a.isSelected,
        isFocused = _a.isFocused,
        node = _a.node,
        decorations = _a.decorations,
        parent = _a.parent,
        readOnly = _a.readOnly;
    var value = editor.value;
    var selection = value.selection,
        schema = value.schema;
    var stack = editor.stack;
    var indexes = node.getSelectionIndexes(selection, isSelected);
    var decs = (0, _concat.default)(decorations).call(decorations, node.getDecorations(stack));
    var childrenDecorations = (0, _getChildrenDecorations.default)(node, decs);
    var children = [];
    (0, _forEach.default)(_context = node.nodes).call(_context, function (child, i) {
      var isChildSelected = !!indexes && indexes.start <= i && i < indexes.end;
      children.push(_this.renderNode(child, isChildSelected, childrenDecorations[i]));
    }); // Attributes that the developer must mix into the element in their
    // custom node renderer component.

    var attributes = {
      "data-key": node.key
    }; // If it's a block node with inline children, add the proper `dir` attribute
    // for text direction.

    if (node.object == "block" && node.nodes.first().object != "block") {
      var direction = node.getTextDirection();
      if (direction == "rtl") attributes.dir = "rtl";
    }

    var props = {
      key: node.key,
      editor: editor,
      isFocused: isFocused,
      isSelected: isSelected,
      node: node,
      parent: parent,
      readOnly: readOnly
    };
    var placeholder = stack.$$find("renderPlaceholder", props);

    if (placeholder) {
      placeholder = /*#__PURE__*/_react.default.cloneElement(placeholder, {
        key: node.key + "-placeholder"
      });
      children = __spreadArrays([placeholder], children);
    }

    var element = stack.$$find("renderNode", __assign(__assign({}, props), {
      attributes: attributes,
      children: children
    }));
    return schema.isVoid(node) ? /*#__PURE__*/_react.default.createElement(_void.default, __assign({}, this.props), element) : element;
  };

  Node.propTypes = {
    block: _slatePropTypes.default.block,
    decorations: _reactImmutableProptypes.default.list.isRequired,
    editor: _propTypes.default.object.isRequired,
    isFocused: _propTypes.default.bool.isRequired,
    isSelected: _propTypes.default.bool.isRequired,
    node: _slatePropTypes.default.node.isRequired,
    parent: _slatePropTypes.default.node.isRequired,
    readOnly: _propTypes.default.bool.isRequired
  };
  return Node;
}(_react.default.Component);
/**
 * Export.
 *
 * @type {Component}
 */


var _default = Node;
exports.default = _default;