"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _create = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/create"));

var _setPrototypeOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/set-prototype-of"));

var _debug = _interopRequireDefault(require("debug"));

var _reactImmutableProptypes = _interopRequireDefault(require("react-immutable-proptypes"));

var _react = _interopRequireDefault(require("react"));

var _slatePropTypes = _interopRequireDefault(require("@zykj/slate-prop-types"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _slate = require("@zykj/slate");

var _leaf = _interopRequireDefault(require("./leaf"));

var __extends = void 0 && (void 0).__extends || function () {
  var extendStatics = function (d, b) {
    extendStatics = _setPrototypeOf.default || {
      __proto__: []
    } instanceof Array && function (d, b) {
      d.__proto__ = b;
    } || function (d, b) {
      for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    };

    return extendStatics(d, b);
  };

  return function (d, b) {
    extendStatics(d, b);

    function __() {
      this.constructor = d;
    }

    d.prototype = b === null ? (0, _create.default)(b) : (__.prototype = b.prototype, new __());
  };
}();

var __spreadArrays = void 0 && (void 0).__spreadArrays || function () {
  for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;

  for (var r = Array(s), k = 0, i = 0; i < il; i++) for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++) r[k] = a[j];

  return r;
};

/**
 * Debug.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:node");
/**
 * Text.
 *
 * @type {Component}
 */

var Text =
/** @class */
function (_super) {
  __extends(Text, _super);

  function Text() {
    /**
     * Property types.
     *
     * @type {Object}
     */
    var _this = _super !== null && _super.apply(this, arguments) || this;
    /**
     * Debug.
     *
     * @param {String} message
     * @param {Mixed} ...args
     */


    _this.debug = function (message) {
      var args = [];

      for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
      }

      var node = _this.props.node;
      var key = node.key;
      debug.apply(void 0, __spreadArrays([message, key + " (text)"], args));
    };
    /**
     * Render a single leaf given a `leaf` and `offset`.
     *
     * @param {List<Leaf>} leaves
     * @param {Leaf} leaf
     * @param {Number} index
     * @param {Number} offset
     * @return {Element} leaf
     */


    _this.renderLeaf = function (leaves, leaf, index, offset) {
      var _a = _this.props,
          block = _a.block,
          node = _a.node,
          parent = _a.parent,
          editor = _a.editor;
      var text = leaf.text,
          marks = leaf.marks;
      return /*#__PURE__*/_react.default.createElement(_leaf.default, {
        key: node.key + "-" + index,
        block: block,
        editor: editor,
        index: index,
        marks: marks,
        node: node,
        offset: offset,
        parent: parent,
        leaves: leaves,
        text: text
      });
    };

    return _this;
  }
  /**
   * Should the node update?
   *
   * @param {Object} nextProps
   * @param {Object} value
   * @return {Boolean}
   */


  Text.prototype.shouldComponentUpdate = function (nextProps) {
    var props = this.props;
    var n = nextProps;
    var p = props; // If the node has changed, update. PERF: There are cases where it will have
    // changed, but it's properties will be exactly the same (eg. copy-paste)
    // which this won't catch. But that's rare and not a drag on performance, so
    // for simplicity we just let them through.

    if (n.node != p.node) return true; // If the node parent is a block node, and it was the last child of the
    // block, re-render to cleanup extra `\n`.

    if (n.parent.object == "block") {
      var pLast = p.parent.nodes.last();
      var nLast = n.parent.nodes.last();
      if (p.node == pLast && n.node != nLast) return true;
    } // Re-render if the current decorations have changed.


    if (!n.decorations.equals(p.decorations)) return true; // Otherwise, don't update.

    return false;
  };
  /**
   * Render.
   *
   * @return {Element}
   */


  Text.prototype.render = function () {
    var _this = this;

    this.debug("render", this);
    var _a = this.props,
        decorations = _a.decorations,
        editor = _a.editor,
        node = _a.node,
        style = _a.style;
    var value = editor.value;
    var document = value.document;
    var key = node.key;
    var decs = (0, _filter.default)(decorations).call(decorations, function (d) {
      var start = d.start,
          end = d.end; // If either of the decoration's keys match, include it.

      if (start.key === key || end.key === key) return true; // Otherwise, if the decoration is in a single node, it's not ours.

      if (start.key === end.key) return false; // If the node's path is before the start path, ignore it.

      var path = document.assertPath(key);
      if (_slate.PathUtils.compare(path, start.path) === -1) return false; // If the node's path is after the end path, ignore it.

      if (_slate.PathUtils.compare(path, end.path) === 1) return false; // Otherwise, include it.

      return true;
    }); // PERF: Take advantage of cache by avoiding arguments

    var leaves = decs.size === 0 ? node.getLeaves() : node.getLeaves(decs);
    var offset = 0;
    var children = (0, _map.default)(leaves).call(leaves, function (leaf, i) {
      var child = _this.renderLeaf(leaves, leaf, i, offset);

      offset += leaf.text.length;
      return child;
    });
    return /*#__PURE__*/_react.default.createElement("span", {
      "data-key": key,
      style: style
    }, children);
  };

  Text.propTypes = {
    block: _slatePropTypes.default.block,
    decorations: _reactImmutableProptypes.default.list.isRequired,
    editor: _propTypes.default.object.isRequired,
    node: _slatePropTypes.default.node.isRequired,
    parent: _slatePropTypes.default.node.isRequired,
    style: _propTypes.default.object
  };
  /**
   * Default prop types.
   *
   * @type {Object}
   */

  Text.defaultProps = {
    style: null
  };
  return Text;
}(_react.default.Component);
/**
 * Export.
 *
 * @type {Component}
 */


var _default = Text;
exports.default = _default;