"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _find = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));

var React = _interopRequireWildcard(require("react"));

var _slateBase64Serializer = _interopRequireDefault(require("@zykj/slate-base64-serializer"));

var _debug = _interopRequireDefault(require("debug"));

var _slatePlainSerializer = _interopRequireDefault(require("@zykj/slate-plain-serializer"));

var _slateDevEnvironment = require("@zykj/slate-dev-environment");

var _getWindow = _interopRequireDefault(require("get-window"));

var _slate = require("@zykj/slate");

var _slateHotkeys = _interopRequireDefault(require("@zykj/slate-hotkeys"));

var _content = _interopRequireDefault(require("../components/content"));

var _cloneFragment = _interopRequireDefault(require("../utils/clone-fragment"));

var _findDomNode = _interopRequireDefault(require("../utils/find-dom-node"));

var _findNode = _interopRequireDefault(require("../utils/find-node"));

var _findPoint = _interopRequireDefault(require("../utils/find-point"));

var _findRange = _interopRequireDefault(require("../utils/find-range"));

var _getEventRange = _interopRequireDefault(require("../utils/get-event-range"));

var _getEventTransfer = _interopRequireDefault(require("../utils/get-event-transfer"));

var _setEventTransfer = _interopRequireDefault(require("../utils/set-event-transfer"));

var __assign = void 0 && (void 0).__assign || function () {
  __assign = _assign.default || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

/**
 * Debug.
 *
 * @type {Function}
 */
var debug = (0, _debug.default)("slate:after");
/**
 * The after plugin.
 *
 * @return {Object}
 */

function AfterPlugin() {
  var isDraggingInternally = null;
  /**
   * On before input, correct any browser inconsistencies.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */

  function onBeforeInput(event, change, editor) {
    debug("onBeforeInput", {
      event: event
    });
    var isSynthetic = !!event.nativeEvent; // If the event is synthetic, it's React's polyfill of `beforeinput` that
    // isn't a true `beforeinput` event with meaningful information. It only
    // gets triggered for character insertions, so we can just insert directly.

    if (isSynthetic) {
      event.preventDefault();
      change.insertText(event.data);
      return;
    } // Otherwise, we can use the information in the `beforeinput` event to
    // figure out the exact change that will occur, and prevent it.


    var targetRange = event.getTargetRanges()[0];
    if (!targetRange) return;
    event.preventDefault();
    var value = change.value;
    var selection = value.selection;
    var range = (0, _findRange.default)(targetRange, value);

    switch (event.inputType) {
      case "deleteByDrag":
      case "deleteByCut":
      case "deleteContent":
      case "deleteContentBackward":
      case "deleteContentForward":
        {
          change.deleteAtRange(range);
          return;
        }

      case "deleteWordBackward":
        {
          change.deleteWordBackwardAtRange(range);
          return;
        }

      case "deleteWordForward":
        {
          change.deleteWordForwardAtRange(range);
          return;
        }

      case "deleteSoftLineBackward":
      case "deleteHardLineBackward":
        {
          change.deleteLineBackwardAtRange(range);
          return;
        }

      case "deleteSoftLineForward":
      case "deleteHardLineForward":
        {
          change.deleteLineForwardAtRange(range);
          return;
        }

      case "insertLineBreak":
      case "insertParagraph":
        {
          if (change.value.isInVoid) {
            change.moveToStartOfNextText();
          } else {
            change.splitBlockAtRange(range);
          }

          return;
        }

      case "insertFromYank":
      case "insertReplacementText":
      case "insertText":
        {
          // COMPAT: `data` should have the text for the `insertText` input type
          // and `dataTransfer` should have the text for the
          // `insertReplacementText` input type, but Safari uses `insertText` for
          // spell check replacements and sets `data` to `null`. (2018/08/09)
          var text = event.data == null ? event.dataTransfer.getData("text/plain") : event.data;
          if (text == null) return;
          change.insertTextAtRange(range, text, selection.marks); // If the text was successfully inserted, and the selection had marks
          // on it, unset the selection's marks.

          if (selection.marks && value.document != change.value.document) {
            change.select({
              marks: null
            });
          }

          return;
        }
    }
  }
  /**
   * On blur.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onBlur(event, change, editor) {
    debug("onBlur", {
      event: event
    });
    change.blur();
  }
  /**
   * On click.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onClick(event, change, editor) {
    if (editor.props.readOnly) return true;
    var value = change.value;
    var document = value.document,
        schema = value.schema;
    var node = (0, _findNode.default)(event.target, value);
    var ancestors = document.getAncestors(node.key);
    var isVoid = node && (schema.isVoid(node) || (0, _some.default)(ancestors).call(ancestors, function (a) {
      return schema.isVoid(a);
    }));

    if (isVoid) {
      // COMPAT: In Chrome & Safari, selections that are at the zero offset of
      // an inline node will be automatically replaced to be at the last offset
      // of a previous inline node, which screws us up, so we always want to set
      // it to the end of the node. (2016/11/29)
      change.focus().moveToEndOfNode(node);
    }

    debug("onClick", {
      event: event
    });
  }
  /**
   * On copy.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCopy(event, change, editor) {
    debug("onCopy", {
      event: event
    });
    (0, _cloneFragment.default)(event, change.value);
  }
  /**
   * On cut.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onCut(event, change, editor) {
    debug("onCut", {
      event: event
    }); // Once the fake cut content has successfully been added to the clipboard,
    // delete the content in the current selection.

    (0, _cloneFragment.default)(event, change.value, change.value.fragment, function () {
      // If user cuts a void block node or a void inline node,
      // manually removes it since selection is collapsed in this case.
      var value = change.value;
      var endBlock = value.endBlock,
          endInline = value.endInline,
          selection = value.selection,
          schema = value.schema;
      var isCollapsed = selection.isCollapsed;
      var isVoidBlock = endBlock && schema.isVoid(endBlock) && isCollapsed;
      var isVoidInline = endInline && schema.isVoid(endInline) && isCollapsed;

      if (isVoidBlock) {
        editor.change(function (c) {
          return c.removeNodeByKey(endBlock.key);
        });
      } else if (isVoidInline) {
        editor.change(function (c) {
          return c.removeNodeByKey(endInline.key);
        });
      } else {
        editor.change(function (c) {
          return c.delete();
        });
      }
    });
  }
  /**
   * On drag end.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragEnd(event, change, editor) {
    debug("onDragEnd", {
      event: event
    });
    isDraggingInternally = null;
  }
  /**
   * On drag over.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragOver(event, change, editor) {
    debug("onDragOver", {
      event: event
    });
  }
  /**
   * On drag start.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDragStart(event, change, editor) {
    var _context;

    debug("onDragStart", {
      event: event
    });
    isDraggingInternally = true;
    var value = change.value;
    var document = value.document,
        schema = value.schema;
    var node = (0, _findNode.default)(event.target, value);
    var ancestors = document.getAncestors(node.key);
    var isVoid = node && (schema.isVoid(node) || (0, _some.default)(ancestors).call(ancestors, function (a) {
      return schema.isVoid(a);
    }));
    var selectionIncludesNode = (0, _some.default)(_context = value.blocks).call(_context, function (block) {
      return block.key === node.key;
    }); // If a void block is dragged and is not selected, select it (necessary for local drags).

    if (isVoid && !selectionIncludesNode) {
      change.moveToRangeOfNode(node);
    }

    var fragment = change.value.fragment;

    var encoded = _slateBase64Serializer.default.serializeNode(fragment);

    (0, _setEventTransfer.default)(event, "fragment", encoded);
  }
  /**
   * On drop.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onDrop(event, change, editor) {
    debug("onDrop", {
      event: event
    });
    var value = change.value;
    var document = value.document,
        selection = value.selection,
        schema = value.schema;
    var window = (0, _getWindow.default)(event.target);
    var target = (0, _getEventRange.default)(event, value);
    if (!target) return;
    var transfer = (0, _getEventTransfer.default)(event);
    var type = transfer.type,
        fragment = transfer.fragment,
        text = transfer.text;
    change.focus(); // If the drag is internal and the target is after the selection, it
    // needs to account for the selection's content being deleted.

    if (isDraggingInternally && selection.end.key == target.end.key && selection.end.offset < target.end.offset) {
      target = target.moveForward(selection.start.key == selection.end.key ? 0 - selection.end.offset + selection.start.offset : 0 - selection.end.offset);
    }

    if (isDraggingInternally) {
      change.delete();
    }

    change.select(target);

    if (type == "text" || type == "html") {
      var anchor = target.anchor;
      var hasVoidParent = document.hasVoidParent(anchor.key, schema);

      if (hasVoidParent) {
        var n = document.getNode(anchor.key);

        while (hasVoidParent) {
          n = document.getNextText(n.key);
          if (!n) break;
          hasVoidParent = document.hasVoidParent(n.key, schema);
        }

        if (n) change.moveToStartOfNode(n);
      }

      if (text) {
        var _context2;

        (0, _forEach.default)(_context2 = text.split("\n")).call(_context2, function (line, i) {
          if (i > 0) change.splitBlock();
          change.insertText(line);
        });
      }
    }

    if (type == "fragment") {
      change.insertFragment(fragment);
    } // COMPAT: React's onSelect event breaks after an onDrop event
    // has fired in a node: https://github.com/facebook/react/issues/11379.
    // Until this is fixed in React, we dispatch a mouseup event on that
    // DOM node, since that will make it go back to normal.


    var focusNode = document.getNode(target.focus.key);
    var el = (0, _findDomNode.default)(focusNode, window);
    if (!el) return;
    el.dispatchEvent(new MouseEvent("mouseup", {
      view: window,
      bubbles: true,
      cancelable: true
    }));
  }
  /**
   * On input.
   *
   * @param {Event} eventvent
   * @param {Change} change
   */


  function onInput(event, change, editor) {
    debug("onInput", {
      event: event
    });
    var window = (0, _getWindow.default)(event.target);
    var value = change.value; // Get the selection point.

    var native = window.getSelection();
    var anchorNode = native.anchorNode;
    var point = (0, _findPoint.default)(anchorNode, 0, value);
    if (!point) return; // Get the text node and leaf in question.

    var document = value.document,
        selection = value.selection;
    var node = document.getDescendant(point.key);
    var block = document.getClosestBlock(node.key);
    var leaves = node.getLeaves();
    var lastText = block.getLastText();
    var lastLeaf = leaves.last();
    var start = 0;
    var end = 0;
    var leaf = (0, _find.default)(leaves).call(leaves, function (r) {
      start = end;
      end += r.text.length;
      if (end > point.offset) return true;
    }) || lastLeaf; // Get the text information.

    var text = leaf.text;
    var textContent = anchorNode.textContent;
    var isLastText = node == lastText;
    var isLastLeaf = leaf == lastLeaf;
    var lastChar = textContent.charAt(textContent.length - 1); // COMPAT: If this is the last leaf, and the DOM text ends in a new line,
    // we will have added another new line in <Leaf>'s render method to account
    // for browsers collapsing a single trailing new lines, so remove it.

    if (isLastText && isLastLeaf && lastChar == "\n") {
      textContent = (0, _slice.default)(textContent).call(textContent, 0, -1);
    } // If the text is no different, abort.


    if (textContent == text) return; // Determine what the selection should be after changing the text.

    var delta = textContent.length - text.length;
    var corrected = selection.moveToEnd().moveForward(delta);
    var entire = selection.moveAnchorTo(point.key, start).moveFocusTo(point.key, end);
    entire = document.resolveRange(entire); // Change the current value to have the leaf's text replaced.

    change.insertTextAtRange(entire, textContent, leaf.marks).select(corrected);
  }
  /**
   * On key down.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onKeyDown(event, change, editor) {
    debug("onKeyDown", {
      event: event
    });
    var value = change.value;
    var schema = value.schema; // COMPAT: In iOS, some of these hotkeys are handled in the
    // `onNativeBeforeInput` handler of the `<Content>` component in order to
    // preserve native autocorrect behavior, so they shouldn't be handled here.

    if (_slateHotkeys.default.isSplitBlock(event) && !_slateDevEnvironment.IS_IOS) {
      return value.isInVoid ? change.moveToStartOfNextText() : change.splitBlock();
    }

    if (_slateHotkeys.default.isDeleteBackward(event) && !_slateDevEnvironment.IS_IOS) {
      return change.deleteCharBackward();
    }

    if (_slateHotkeys.default.isDeleteForward(event) && !_slateDevEnvironment.IS_IOS) {
      return change.deleteCharForward();
    }

    if (_slateHotkeys.default.isDeleteLineBackward(event)) {
      return change.deleteLineBackward();
    }

    if (_slateHotkeys.default.isDeleteLineForward(event)) {
      return change.deleteLineForward();
    }

    if (_slateHotkeys.default.isDeleteWordBackward(event)) {
      return change.deleteWordBackward();
    }

    if (_slateHotkeys.default.isDeleteWordForward(event)) {
      return change.deleteWordForward();
    }

    if (_slateHotkeys.default.isRedo(event)) {
      return change.redo();
    }

    if (_slateHotkeys.default.isUndo(event)) {
      return change.undo();
    } // COMPAT: Certain browsers don't handle the selection updates properly. In
    // Chrome, the selection isn't properly extended. And in Firefox, the
    // selection isn't properly collapsed. (2017/10/17)


    if (_slateHotkeys.default.isMoveLineBackward(event)) {
      event.preventDefault();
      return change.moveToStartOfBlock();
    }

    if (_slateHotkeys.default.isMoveLineForward(event)) {
      event.preventDefault();
      return change.moveToEndOfBlock();
    }

    if (_slateHotkeys.default.isExtendLineBackward(event)) {
      event.preventDefault();
      return change.moveFocusToStartOfBlock();
    }

    if (_slateHotkeys.default.isExtendLineForward(event)) {
      event.preventDefault();
      return change.moveFocusToEndOfBlock();
    } // COMPAT: If a void node is selected, or a zero-width text node adjacent to
    // an inline is selected, we need to handle these hotkeys manually because
    // browsers won't know what to do.


    if (_slateHotkeys.default.isMoveBackward(event)) {
      var document_1 = value.document,
          isInVoid = value.isInVoid,
          previousText = value.previousText,
          startText = value.startText;
      var isPreviousInVoid = previousText && document_1.hasVoidParent(previousText.key, schema);

      if (isInVoid || isPreviousInVoid || startText.text == "") {
        event.preventDefault();
        return change.moveBackward();
      }
    }

    if (_slateHotkeys.default.isMoveForward(event)) {
      var document_2 = value.document,
          isInVoid = value.isInVoid,
          nextText = value.nextText,
          startText = value.startText;
      var isNextInVoid = nextText && document_2.hasVoidParent(nextText.key, schema);

      if (isInVoid || isNextInVoid || startText.text == "") {
        event.preventDefault();
        return change.moveForward();
      }
    }

    if (_slateHotkeys.default.isExtendBackward(event)) {
      var document_3 = value.document,
          isInVoid = value.isInVoid,
          previousText = value.previousText,
          startText = value.startText;
      var isPreviousInVoid = previousText && document_3.hasVoidParent(previousText.key, schema);

      if (isInVoid || isPreviousInVoid || startText.text == "") {
        event.preventDefault();
        return change.moveFocusBackward();
      }
    }

    if (_slateHotkeys.default.isExtendForward(event)) {
      var document_4 = value.document,
          isInVoid = value.isInVoid,
          nextText = value.nextText,
          startText = value.startText;
      var isNextInVoid = nextText && document_4.hasVoidParent(nextText.key, schema);

      if (isInVoid || isNextInVoid || startText.text == "") {
        event.preventDefault();
        return change.moveFocusForward();
      }
    }
  }
  /**
   * On paste.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onPaste(event, change, editor) {
    debug("onPaste", {
      event: event
    });
    var transfer = (0, _getEventTransfer.default)(event);
    var type = transfer.type,
        fragment = transfer.fragment,
        text = transfer.text;

    if (type == "fragment") {
      change.insertFragment(fragment);
    }

    if (type == "text" || type == "html") {
      if (!text) return;
      var value = change.value;
      var document_5 = value.document,
          selection = value.selection,
          startBlock = value.startBlock,
          schema = value.schema;
      if (schema.isVoid(startBlock)) return;
      var defaultBlock = startBlock;
      var defaultMarks = document_5.getInsertMarksAtRange(selection);

      var frag = _slatePlainSerializer.default.deserialize(text, {
        defaultBlock: defaultBlock,
        defaultMarks: defaultMarks
      }).document;

      change.insertFragment(frag);
    }
  }
  /**
   * On select.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   */


  function onSelect(event, change, editor) {
    debug("onSelect", {
      event: event
    });
    var window = (0, _getWindow.default)(event.target);
    var value = change.value;
    var document = value.document,
        schema = value.schema;
    var native = window.getSelection(); // If there are no ranges, the editor was blurred natively.

    if (!native.rangeCount) {
      change.blur();
      return;
    } // Otherwise, determine the Slate selection from the native one.


    var range = (0, _findRange.default)(native, value);
    if (!range) return;
    var anchor = range.anchor,
        focus = range.focus;
    var anchorText = document.getNode(anchor.key);
    var focusText = document.getNode(focus.key);
    var anchorInline = document.getClosestInline(anchor.key);
    var focusInline = document.getClosestInline(focus.key);
    var focusBlock = document.getClosestBlock(focus.key);
    var anchorBlock = document.getClosestBlock(anchor.key); // COMPAT: If the anchor point is at the start of a non-void, and the
    // focus point is inside a void node with an offset that isn't `0`, set
    // the focus offset to `0`. This is due to void nodes <span>'s being
    // positioned off screen, resulting in the offset always being greater
    // than `0`. Since we can't know what it really should be, and since an
    // offset of `0` is less destructive because it creates a hanging
    // selection, go with `0`. (2017/09/07)

    if (anchorBlock && !schema.isVoid(anchorBlock) && anchor.offset == 0 && focusBlock && schema.isVoid(focusBlock) && focus.offset != 0) {
      range = range.setFocus(focus.setOffset(0));
    } // COMPAT: If the selection is at the end of a non-void inline node, and
    // there is a node after it, put it in the node after instead. This
    // standardizes the behavior, since it's indistinguishable to the user.


    if (anchorInline && !schema.isVoid(anchorInline) && anchor.offset == anchorText.text.length) {
      var block = document.getClosestBlock(anchor.key);
      var next = block.getNextText(anchor.key);
      if (next) range = range.moveAnchorTo(next.key, 0);
    }

    if (focusInline && !focusInline.isVoid && focus.offset == focusText.text.length) {
      var block = document.getClosestBlock(focus.key);
      var next = block.getNextText(focus.key);
      if (next) range = range.moveFocusTo(next.key, 0);
    }

    var selection = document.createSelection(range);
    selection = selection.setIsFocused(true);
    change.select(selection);
  }
  /**
   * Render editor.
   *
   * @param {Object} props
   * @param {Editor} editor
   * @return {Object}
   */


  function renderEditor(props, editor) {
    var handlers = editor.handlers;
    return /*#__PURE__*/React.createElement(_content.default, __assign({}, handlers, {
      autoCorrect: props.autoCorrect,
      className: props.className,
      children: props.children,
      editor: editor,
      readOnly: props.readOnly,
      role: props.role,
      spellCheck: props.spellCheck,
      style: props.style,
      tabIndex: props.tabIndex,
      tagName: props.tagName
    }));
  }
  /**
   * Render node.
   *
   * @param {Object} props
   * @return {Element}
   */


  function renderNode(props) {
    var attributes = props.attributes,
        children = props.children,
        node = props.node;
    if (node.object != "block" && node.object != "inline") return;
    var Tag = node.object == "block" ? "div" : "span";
    var style = {
      position: "relative"
    };
    return /*#__PURE__*/React.createElement(Tag, __assign({}, attributes, {
      style: style
    }), children);
  }
  /**
   * Render placeholder.
   *
   * @param {Object} props
   * @return {Element}
   */


  function renderPlaceholder(props) {
    var editor = props.editor,
        node = props.node;
    if (!editor.props.placeholder) return;
    if (editor.state.isComposing) return;
    if (node.object != "block") return;
    if (!_slate.Text.isTextList(node.nodes)) return;
    if (node.text != "") return;
    if (editor.value.document.getBlocks().size > 1) return;
    var style = {
      pointerEvents: "none",
      display: "inline-block",
      width: "0",
      maxWidth: "100%",
      whiteSpace: "nowrap",
      opacity: "0.333"
    };
    return /*#__PURE__*/React.createElement("span", {
      contentEditable: false,
      style: style
    }, editor.props.placeholder);
  }
  /**
   * Return the plugin.
   *
   * @type {Object}
   */


  return {
    onBeforeInput: onBeforeInput,
    onBlur: onBlur,
    onClick: onClick,
    onCopy: onCopy,
    onCut: onCut,
    onDragEnd: onDragEnd,
    onDragOver: onDragOver,
    onDragStart: onDragStart,
    onDrop: onDrop,
    onInput: onInput,
    onKeyDown: onKeyDown,
    onPaste: onPaste,
    onSelect: onSelect,
    renderEditor: renderEditor,
    renderNode: renderNode,
    renderPlaceholder: renderPlaceholder
  };
}
/**
 * Export.
 *
 * @type {Object}
 */


var _default = AfterPlugin;
exports.default = _default;