"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _getWindow = _interopRequireDefault(require("get-window"));

var _slateDevEnvironment = require("@zykj/slate-dev-environment");

var _findPoint = _interopRequireDefault(require("./find-point"));

var _findDomPoint = _interopRequireDefault(require("./find-dom-point"));

/**
 * Find a Slate range from a DOM `native` selection.
 *
 * @param {Selection} native
 * @param {Value} value
 * @return {Range}
 */
function findRange(native, value) {
  var el = native.anchorNode || native.startContainer;
  if (!el) return null;
  var window = (0, _getWindow.default)(el); // If the `native` object is a DOM `Range` or `StaticRange` object, change it
  // into something that looks like a DOM `Selection` instead.

  if (native instanceof window.Range || window.StaticRange && native instanceof window.StaticRange) {
    native = {
      anchorNode: native.startContainer,
      anchorOffset: native.startOffset,
      focusNode: native.endContainer,
      focusOffset: native.endOffset
    };
  }

  var anchorNode = native.anchorNode,
      anchorOffset = native.anchorOffset,
      focusNode = native.focusNode,
      focusOffset = native.focusOffset,
      isCollapsed = native.isCollapsed;
  var anchor = (0, _findPoint.default)(anchorNode, anchorOffset, value);
  var focus = isCollapsed ? anchor : (0, _findPoint.default)(focusNode, focusOffset, value);
  if (!anchor || !focus) return null; // COMPAT: ??? The Edge browser seems to have a case where if you select the
  // last word of a span, it sets the endContainer to the containing span.
  // `selection-is-backward` doesn't handle this case.

  if (_slateDevEnvironment.IS_IE || _slateDevEnvironment.IS_EDGE) {
    var domAnchor = (0, _findDomPoint.default)(anchor);
    var domFocus = (0, _findDomPoint.default)(focus);
    native = {
      anchorNode: domAnchor.node,
      anchorOffset: domAnchor.offset,
      focusNode: domFocus.node,
      focusOffset: domFocus.offset
    };
  }

  var document = value.document;
  var range = document.createRange({
    anchor: anchor,
    focus: focus
  });
  return range;
}
/**
 * Export.
 *
 * @type {Function}
 */


var _default = findRange;
exports.default = _default;