"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _getWindow = _interopRequireDefault(require("get-window"));

var _findNode = _interopRequireDefault(require("./find-node"));

var _findRange = _interopRequireDefault(require("./find-range"));

/**
 * Get the target range from a DOM `event`.
 *
 * @param {Event} event
 * @param {Value} value
 * @return {Range}
 */
function getEventRange(event, value) {
  if (event.nativeEvent) {
    event = event.nativeEvent;
  }

  var x = event.x,
      y = event.y,
      target = event.target;
  if (x == null || y == null) return null;
  var document = value.document,
      schema = value.schema;
  var node = (0, _findNode.default)(target, value);
  if (!node) return null; // If the drop target is inside a void node, move it into either the next or
  // previous node, depending on which side the `x` and `y` coordinates are
  // closest to.

  if (schema.isVoid(node)) {
    var rect = target.getBoundingClientRect();
    var isPrevious = node.object == "inline" ? x - rect.left < rect.left + rect.width - x : y - rect.top < rect.top + rect.height - y;
    var text = node.getFirstText();
    var range_1 = document.createRange();

    if (isPrevious) {
      var previousText = document.getPreviousText(text.key);

      if (previousText) {
        return range_1.moveToEndOfNode(previousText);
      }
    }

    var nextText = document.getNextText(text.key);
    return nextText ? range_1.moveToStartOfNode(nextText) : null;
  } // Else resolve a range from the caret position where the drop occured.


  var window = (0, _getWindow.default)(target);
  var native; // COMPAT: In Firefox, `caretRangeFromPoint` doesn't exist. (2016/07/25)

  if (window.document.caretRangeFromPoint) {
    native = window.document.caretRangeFromPoint(x, y);
  } else if (window.document.caretPositionFromPoint) {
    var position = window.document.caretPositionFromPoint(x, y);
    native = window.document.createRange();
    native.setStart(position.offsetNode, position.offset);
    native.setEnd(position.offsetNode, position.offset);
  } else if (window.document.body.createTextRange) {
    // COMPAT: In IE, `caretRangeFromPoint` and
    // `caretPositionFromPoint` don't exist. (2018/07/11)
    native = window.document.body.createTextRange();

    try {
      native.moveToPoint(x, y);
    } catch (error) {
      // IE11 will raise an `unspecified error` if `moveToPoint` is
      // called during a dropEvent.
      return null;
    }
  } // Resolve a Slate range from the DOM range.


  var range = (0, _findRange.default)(native, value);
  if (!range) return null;
  return range;
}
/**
 * Export.
 *
 * @type {Function}
 */


var _default = getEventRange;
exports.default = _default;